// SPDX-License-Identifier: MIT
pragma solidity ^0.8.0;

import "../lib/MetaKeepLambdaConstants.sol";

abstract contract MetaKeepLambdaEIP712Base is MetaKeepLambdaConstants {
    bytes32 public domainSeperator;

    function _setDomainSeperator(string memory name) internal {
        require(
            domainSeperator == bytes32(0),
            "MetaKeepLambda: Domain seperator already set"
        );
        domainSeperator = keccak256(
            abi.encode(
                EIP712_DOMAIN_TYPEHASH,
                keccak256(bytes(name)),
                keccak256(bytes(ERC712_VERSION)),
                address(this),
                bytes32(getChainId())
            )
        );
    }

    //slither-disable-next-line assembly
    function getChainId() private view returns (uint256) {
        uint256 id;
        assembly {
            id := chainid()
        }
        return id;
    }

    /**
     * Accept message hash and returns hash message in EIP712 compatible form
     * So that it can be used to recover signer from signature signed using EIP712 formatted data
     * https://eips.ethereum.org/EIPS/eip-712
     * "\\x19" makes the encoding deterministic
     * "\\x01" is the version byte to make it compatible to EIP-191
     */
    function toTypedMessageHash(bytes32 messageHash)
        internal
        view
        returns (bytes32)
    {
        return
            keccak256(
                abi.encodePacked("\x19\x01", domainSeperator, messageHash)
            );
    }
}
